package com.ccwangluo.ccim;


import com.ccwangluo.ccim.listener.ResultCallback;
import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.ccwangluo.ccim.manage.CallBackManager;
import com.ccwangluo.ccim.manage.SocketManageer;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import ccim.protocol.Chat;
import ccim.protocol.Message;
import ccim.protocol.MessageSpec;
import ccim.protocol.Push;

public class CCIMClient {

    private final Logger logger = Logger.getLogger(CCIMClient.class.getName());

    private SocketManageer socketManageer;
    private CallBackManager<Chat.ChatResponseMessage> callBackManager;

    private CCIMClient() {
    }

    private static CCIMClient mInstance;

    public static CCIMClient getInstance() {
        if (mInstance == null) {
            synchronized (CCIMClient.class) {
                if (mInstance == null) {
                    mInstance = new CCIMClient();
                }
            }
        }
        return mInstance;
    }

    public void initSDK() {
        callBackManager = new CallBackManager<>();
        socketManageer = new SocketManageer(callBackManager);
        socketManageer.initAuth();
    }


    public void registerMessageListener(ResultEventCallback<Chat.ChatResponseMessage> resultEventCallback) {
        callBackManager.addCallBack(0, chatResponseMessage -> {
            logger.info("registerMessageListener : " + chatResponseMessage);
            if (chatResponseMessage.hasPushPrivateMsgRes()) {
                MessageSpec.ChatMessage pushPrivateMsgRes = chatResponseMessage.getPushPrivateMsgRes().getMessage();
                pushPrivateMessageAck(pushPrivateMsgRes.getSenderUserId(), pushPrivateMsgRes.getMsgId());
            }
            if (chatResponseMessage.hasPushGroupMsgRes()) {
                Push.PushGroupMessageResponse message = chatResponseMessage.getPushGroupMsgRes();
                pushGroupMessageAck(message.getGroupId(), message.getMessage().getMsgId());
            }
            if (chatResponseMessage.hasPushChannelMsgRes()) {
                Push.PushChannelMessageResponse channelMsgRes = chatResponseMessage.getPushChannelMsgRes();
                pushChannelMessage(channelMsgRes.getChannelId(), channelMsgRes.getMessage().getMsgId());
            }
            resultEventCallback.onSuccess(chatResponseMessage);
        });
    }


    public void login(String username, String password,Long appId, ResultCallback resultEventCallback) {
        socketManageer.login(username, password, appId,new ResultEventCallback<String>() {
            @Override
            public void onSuccess(String s) {
                resultEventCallback.onSuccess();
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });
    }


    /**
     * 获取联系人列表
     */
    public synchronized void getChatList(ResultEventCallback<Chat.PullContactsResponse> resultEventCallback) {
        Chat.PullContactsRequest builder = Chat.PullContactsRequest.newBuilder()
                .build();
        int id = socketManageer.sendRequest(builder);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullContactsRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.PullContactsResponse pullContactsRes = chatResponseMessage.getPullContactsRes();
                    resultEventCallback.onSuccess(pullContactsRes);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });

    }


    /**
     * 同步离线消息
     */
    public void getOfflineMessage(ResultEventCallback<ArrayList<Message.PullOfflineMessageResponse.OfflineMessage>> resultEventCallback) {
        ArrayList<Message.PullOfflineMessageResponse.OfflineMessage> offlineMessages = new ArrayList<>();
        Message.PullOfflineMessageRequest offlineMessageRequest = Message.PullOfflineMessageRequest.newBuilder().build();
        int id = socketManageer.sendRequest( offlineMessageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullOfflineMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    List<Message.PullOfflineMessageResponse.OfflineMessage> offlineMsgsList = chatResponseMessage.getPullOfflineMsgRes().getOfflineMsgsList();
                    offlineMessages.addAll(offlineMsgsList);
                    if (offlineMsgsList.isEmpty()) {
                        resultEventCallback.onSuccess(offlineMessages);
                        callBackManager.removeCallBack(chatResponseMessage.getHeader().getSeq());
                    }
                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });

    }

    /**
     * 搜索好友
     */
    public void searchUser(String userName, ResultEventCallback<List<Chat.SearchResponse.User>> resultEventCallback) {
        Chat.SearchRequest searchRequest = Chat.SearchRequest.newBuilder().setUsername(userName).build();
        int id = socketManageer.sendRequest(searchRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSearchRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    List<Chat.SearchResponse.User> searchRes = chatResponseMessage.getSearchRes().getUsersList();
                    resultEventCallback.onSuccess(searchRes);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });
    }

    /**
     * 添加好友
     */
    public void addContact(Long userId, ResultEventCallback<Chat.AddContactResponse.AddResult> callback) {
        Chat.AddContactRequest addContactRequest = Chat.AddContactRequest.newBuilder().setUserId(userId).build();
        int id = socketManageer.sendRequest(addContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasAddContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.AddContactResponse addContactRes = chatResponseMessage.getAddContactRes();
                    Chat.AddContactResponse.AddResult result = addContactRes.getResult();
                    callback.onSuccess(result);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });

    }

    /**
     * 联系人申请列表
     */
    public void getContactRequests(ResultEventCallback<Chat.ContactRequestsResponse> callback) {
        Chat.ContactRequestsRequest contactRequestsRequest = Chat.ContactRequestsRequest.newBuilder().build();
        int id = socketManageer.sendRequest(contactRequestsRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasContactRequestsRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.ContactRequestsResponse contactRequestsRes = chatResponseMessage.getContactRequestsRes();
                    callback.onSuccess(contactRequestsRes);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });

    }

    /**
     * 验证联系人申请
     */
    public void checkContactReq(Long reqUserId, boolean accept, ResultCallback callback) {
        Chat.CheckContactReqRequest checkContactReqRequest = Chat.CheckContactReqRequest.newBuilder()
                .setReqUserId(reqUserId)
                .setAccept(accept)
                .build();
        int id = socketManageer.sendRequest(checkContactReqRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCheckContactReqRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 创建联系人群组
     */
    public void createContactGroup(String name, ResultEventCallback<Long> callback) {
        Chat.CreateContactGroupRequest createContactGroupRequest = Chat.CreateContactGroupRequest.newBuilder()
                .setName(name)
                .build();
        int id = socketManageer.sendRequest(createContactGroupRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCreateContactGroupRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.CreateContactGroupResponse createContactGroupRes = chatResponseMessage.getCreateContactGroupRes();
                    callback.onSuccess(createContactGroupRes.getGroupId());
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 移动联系人所在的群组
     */
    public void moveContactGroup(Long groupId, Long userId, ResultCallback callback) {
        Chat.MoveContactGroupRequest moveContactGroupRequest = Chat.MoveContactGroupRequest.newBuilder()
                .setContactGroupId(groupId)
                .setContactUserId(userId)
                .build();
        int id = socketManageer.sendRequest(moveContactGroupRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasMoveContactGroupRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 删除联系人
     */
    public void deleteContact(Long userId, ResultCallback callback) {
        Chat.DeleteContactRequest deleteContactRequest = Chat.DeleteContactRequest.newBuilder()
                .setContactUserId(userId)
                .build();
        int id = socketManageer.sendRequest(deleteContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasDeleteContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 拉黑联系人
     */
    public void blockContact(Long userId, Boolean mute, ResultCallback callback) {
        Chat.BlockContactRequest blockContactRequest = Chat.BlockContactRequest.newBuilder()
                .setContactUserId(userId)
                .setMute(mute)
                .build();
        int id = socketManageer.sendRequest(blockContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasBlockContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 举报违规用户
     */
    public void reportIllegalUser(String title, String content, Long userId, ResultCallback callback) {
        Chat.ReportIllegalUserRequest reportIllegalUserRequest = Chat.ReportIllegalUserRequest.newBuilder()
                .setContent(content)
                .setIllegalUserId(userId)
                .setTitle(title)
                .build();
        int id = socketManageer.sendRequest(reportIllegalUserRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasReportIllegalUserRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 发送消息
     */
    public void sendTextMessage(String content, Long reciveUserId, ResultCallback callback) {
        Message.SendMessageRequest messageRequest = Message.SendMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.TEXT)
                .setReceiverUserId(reciveUserId).build();
        int id = socketManageer.sendRequest(messageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage.getSearchRes()));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 发送群组消息
     */
    public void sendGroupMessage(String content, Long groupId, ResultCallback callback) {
        Message.SendGroupMessageRequest sendGroupMessageRequest = Message.SendGroupMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.TEXT)
                .setGroupId(groupId)
                .build();
        int id = socketManageer.sendRequest(sendGroupMessageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendGroupMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 发送群组消息
     */
    public void sendChannelMessage(String content, Long channelId, ResultCallback callback) {
        Message.SendChannelMessageRequest sendChannelMessageRequest = Message.SendChannelMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.TEXT)
                .setChannelId(channelId)
                .build();
        int id = socketManageer.sendRequest(sendChannelMessageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendChannelMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 推送私人消息成功ack
     */
    public void pushPrivateMessageAck(Long contactUserId, Long msgId) {
        Push.PushPrivateMessageAckRequest pushPrivateMessageAckRequest = Push.PushPrivateMessageAckRequest.newBuilder()
                .setContactUserId(contactUserId)
                .setMsgId(msgId)
                .build();
        int id = socketManageer.sendRequest(pushPrivateMessageAckRequest);
        callBackManager.addCallBack(id, chatResponseMessage -> logger.info(String.valueOf(chatResponseMessage)));
    }

    /**
     * 推送渠道消息成功ack
     */
    public void pushGroupMessageAck(Long groupId, Long msgId) {
        Push.PushGroupMessageAckRequest groupMessageAckRequest = Push.PushGroupMessageAckRequest.newBuilder()
                .setGroupId(groupId)
                .setMsgId(msgId)
                .build();
        int id = socketManageer.sendRequest(groupMessageAckRequest);
        callBackManager.addCallBack(id, chatResponseMessage -> logger.info(String.valueOf(chatResponseMessage)));
    }

    /**
     * 推送渠道消息成功ack
     */
    public void pushChannelMessage(Long channelId, Long msgId) {
        Push.PushChannelMessageAckRequest channelMessageAckRequest = Push.PushChannelMessageAckRequest.newBuilder()
                .setChannelId(channelId)
                .setMsgId(msgId)
                .build();
        int id = socketManageer.sendRequest(channelMessageAckRequest);
        callBackManager.addCallBack(id, chatResponseMessage -> logger.info(String.valueOf(chatResponseMessage)));
    }

}
